WIDTH = 480
HEIGHT = 640
PREVIEW_WIDTH = 400
PREVIEW_HEIGHT = PREVIEW_WIDTH
APPEARANCE_MODE = 'system'
COLOR_THEME = [
  'src/ezsam/gui/assets/theme.json',
  'site-packages/ezsam/gui/assets/theme.json',
  # 'blue' # A fallback built-in theme
]

DEFAULT_MODEL = 'SAM Large'
DEFAULT_DEBUG = False
